import java.util.Vector;
import org.w3c.dom.*;
import javax.xml.parsers.*;

/**
 * My Tree implemtiert die sondierenden Funktionen von Tree.
 * 
 * Created on Nov 25, 2003
 * 
 * @author David Kaltschmidt & Benjamin Schrter
 */
public class MyTree extends Tree {
	
	/**
	 * Jeder Baum hat einen Wurzelknoten
	 */
	Node root;

	/**
	 * type bestimmt den Typ des Baumes (bei uns type = 2)
	 */
	int type;
	
	/**
	 *  Anzahl der Bltter
	 */
	int size;
	
	/**
	 * Hhe des Baumes
	 */
	int height;
	
	/**
	 * Erzeugt einen Baum indem ein Wurzelknoten initialisiert wird.
	 */
	public MyTree() {
		type = 2;
		size = 0;
		height = 0;
		root = new Node(type, null);
	}
 
	/**
	 * Beschreibt die Gre des Baumes (Anzahl der Bltter).
	 * 
	 * @return Gre des Baumes
	 */
	public int size() {
		return size;
	}

	/**
	 * Sagt aus, ob der Baum leer ist.
	 * Dies ist der Fall, wenn die Gre der Wurzel Null ist.
	 * 
	 * @return Boolean, ob der Baum leer ist
	 */
	public boolean isEmpty() {
		return (size == 0);
	}

	/**
	 * Gibt die Hhe des Baumes zurck.
	 * Die Methode hangelt sich dabei von der Wurzel 
	 * immer ber den ersten Folgeknoten hinab und zhlt
	 * dabei mit.
	 *  
	 * @return Hhe des Baumes
	 */
	public int height() {
		return height;
	}

	/**
	 * Sagt aus, ob der gegebene Schlssel enthalten ist.
	 * 
	 * @see Node#contains(int)
	 * @return Boolean, ob der Schlssel enthalten ist.
	 * @param key Schlssel, der gesucht werden soll
	 */
	public boolean contains(int key) {
		return root.contains(key);
	}
	
	/**
	 * Liefert den kleinsten Schlssel im Baum.
	 * Hangelt sich immer am linkesten Kindknoten hinab
	 * bis zu den Blttern.
	 * 
	 * @throws Exception bei leerem Baum
	 * @return Kleinster Schlssel im Baum
	 */
	public int minimum() throws EmptyTreeException {
		if (isEmpty()) throw new EmptyTreeException("Baum leer -> kein Minimum");
		Node n = this.root;
		while(!n.hasLeaves()) {
			n = n.nodes[0];
		}
		return n.leaves[0];
	}

	/**
	 * Liefert den grten Schlssel im Baum.
	 * Hangelt sich immer ber den rechtesten 
	 * Kindknoten bis zu den Blttern.
	 * 
	 * @throws Exception bei leerem Baum
	 * @return Grter Schlssel im Baum
	 */
	public int maximum() throws EmptyTreeException {
		if (isEmpty()) throw new EmptyTreeException("Baum leer -> kein Maximum");
		Node n = this.root;
		while(!n.hasLeaves()) {
			n = n.nodes[n.size - 1];
		}
		return n.leaves[n.size - 1];
	}
	
	/**
	 * Fgt einen neuen Schlssel ein.
	 * Testet erst, ob Schlssel schon enthalten ist.
	 * Falls nicht, bergebe Schlssel an Wurzelknoten.
	 * 
	 * @param key Schlssel, der eingefgt werden soll.
	 * @see tree.Node#add(int)
	 */
	public void add(int key) {
		System.out.println("Versuche hinzuzufgen: " + key);
		root.add(key, this);
	}
	
	
	/**
	 * Nicht implementiert
	 */
	public boolean remove(int key) {
		return false;
	}

	/**
	 * Lscht den Baum.
	 * Initialisiert eine neue Wurzel.
	 * Damit existieren keine weiteren Referenzen
	 * mehr auf andere Knoten.
	 */
	public void clear() {
		root = new Node(type, null);
	}

	/**
	 * Liefert ein sortiertes Array aller Schlssel des Baumes.
	 * Speichert alle Schlssel in einem Vektor.
	 * Dieser wird dann in ein Array transferiert.
	 * Wegen copy-by-value wird nicht direkt mit einem
	 * int-Array gearbeitet.
	 *  
	 * @see Node#traverse()
	 */
	public int[] toIntArray() {
		Vector v = new Vector(size());
		Node n = this.root;
		n.traverse(v);
		int[] ret = new int[v.size()];
		for(int i = 0; i < ret.length; i++)
			ret[i] = ((Integer) v.get(i)).intValue();
		return ret;
	}

	/**
	 * Liefert Stringrepresentation des Baumes.
	 * 
	 * @see Node#print()
	 */
	public String toString() {
		Vector v = new Vector();
		root.print(v);
		String ret = new String("");
		for(int i = 0; i < v.size(); i++)
			ret += v.get(i);
		return ret;
	}

	/**
	 * Liefert ein XML-Dokument, dass den Baum darstellt.
	 * Benutzt das W3C.DOM-API.
	 * 
	 * @return XML-Document des Baumes
	 * @see Node#toXml()
	 */
	public Document toXml() {
		Document document = null; 
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			document = builder.newDocument();
			
			root.toXml(document,null);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}

		return (document);
	}
}

