import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;


/**
 * Klasse Tree fr einen 2-3-Baum, der ganze Zahlen speichert. 
 * 
 * Diese Klasse enthlt:
 * - sondierende Funktionen, welche Auskunft ber den aktuellen Status des
 *   Objektes geben (das sieht viel aus, ist aber wenig).
 * - Methoden zum einfgen und entfernen (sieht wenig aus, ist aber viel).
 * - Ausgabefunktionen. "toIntArray" ist hilfreich zum Testen, da die Zahlen-
 *   folge streng aufsteigend sortiert sein muss, was sich leicht prfen lsst.
 *   "toString" gibt eine String-Reprsentation des Baumes zurck. Wie die 
 *   aussieht, ist nicht so wichtig. Aber zum Debuggen ist die Funktion 
 *   unverzichtbar.
 * - Testfunktionen. Nach Bedarf erweitern. Hilfreich wre z.B. eine Funktion
 *   "checkNodes", die prft, ob jedes Kind eines Knotens diesen Knoten auch
 *   zum Elter hat.
 * - Die innere Klasse "EmptyTreeException". 
 * 
 * Was ihr machen soll:
 * - Schreibt eine Klasse (z.B. "MyTree"), welche alle abstrakten Methoden
 *   implementiert.
 * - Entwerft eine weitere Klasse (z.B. "Node") fr die inneren Knoten des 
 *   Baumes. Die Blattknoten kann man als Integer-Objekte speichern. Oder 
 *   man speichert sie gleich in den inneren Knoten der vorletzten Ebene als 
 *   int-Werte. 
 * - Setzt den Testlauf fr nichtleere Bume fort. 
 * 
 * Was zu beachten ist:
 * - Die 2-3 Bume der Vorlesung speichern alle Schlssel in den Blttern (dort,
 *   aber nicht nur dort). Achtung: Im Internet finden man meistens eine andere 
 *   Variante. Diese wird nicht akzeptiert.
 * - Implementiert das Einfgen und Entfernen so, wie es in der Vorlesung
 *   vorgestellt wurde.
 * - Verschwendet nicht systematisch Speicherplatz! Verwendet fr die 
 *   2 oder 3 Kinder eines inneren Knotens keinen Vector, verwendet fr die
 *   Blattknoten nicht die gleichen Objekte wie fr die inneren Knoten.
 * - Euer Baum sollte mit (theoretisch) beliebig groen Datenmengen klarkommen,
 *   also keine fest eingebaute Obergrenze haben.
 */
public abstract class Tree {

	///////////////////////// sondierende Funktionen ///////////////////////////

	// bestimmt die Gre des Baumes. Ein leerer Baum hat Gre 0.
	public abstract int size();

	// ist der Baum leer?
	public abstract boolean isEmpty();

	// bestimmt die Hhe des Baumes. Ein leerer Baum hat Hhe -1.
	public abstract int height();

	// ist der Schlssel enthalten?
	public abstract boolean contains(int key);

	// Gibt das Minimum zurck. 
	// Falls der Baum leer ist, wird eine Exception ausgelst.
	public abstract int minimum() throws EmptyTreeException;

	// Gibt das Maximum zurck.
	// Falls der Baum leer ist, wird eine Exception ausgelst.
	public abstract int maximum() throws EmptyTreeException;

	////////////////////////// Aufbauen und abbauen ////////////////////////////

	// fgt einen Schlssel in den Baum ein
	public abstract void add(int key);

	// lscht den angegebenen Schlssel aus dem Baum.
	// Falls der Schlssel gefunden (und gelscht) wird, ist der Rckgabewert
	// true, ansonsten false.
	public abstract boolean remove(int key);

	// leert den Baum
	public abstract void clear();

	///////////////////////////////// Ausgabe ////////////////////////////////// 

	// gibt die im Baum enthaltenen Schlssel in aufsteigender Reihenfolge 
	// zurck.
	public abstract int[] toIntArray();

	// gibt eine Zeichenkette zurck, die den Baum darstellt 
	public abstract String toString();

	///////////////////////////////// Testen ///////////////////////////////////

	public static void main(String[] args) {
		// hier wird der Konstruktor eurer Klasse aufgerufen.

		MyTree tree = new MyTree();
		
		System.out.println("Gre des leeren Baumes: " + tree.size());
		System.out.println("Ist der Baum leer?: " + tree.isEmpty());
		System.out.println("Hhe des leeren Baumes: " + tree.height());
		System.out.println(
			"Ist Schlssel 1 im leeren Baum?: " + tree.contains(1));
		try {
			System.out.print("Minimum im leeren Baum: ");
			System.out.println(tree.minimum());
		} catch (EmptyTreeException e) {
			System.out.println(e.getMessage());
		}
		try {
			System.out.print("Maximum im leeren Baum: ");
			System.out.println(tree.maximum());
		} catch (EmptyTreeException e) {
			System.out.println(e.getMessage());
		}
		System.out.println("Entfernen im leeren Baum: " + tree.remove(2));
		System.out.println(
			"Zahlenfolge im leeren Baum: "
				+ intArrayToString(tree.toIntArray()));
		System.out.println("XML-Ausgabe vom leeren Baum: ");

		System.out.println("+++++++++++ Test durch Zufall ++++++++++++");
		boolean error = false;
		for (int i = 0; i < 100 && !error; ++i) {
			System.out.println("*****Tree No: " + i);
			tree = new MyTree();
			for (int j = 0; j < 1000 && !error; ++j) {
				int number =(int) (Math.random() * 100);
				tree.add(number);
				int[] keys = tree.toIntArray();
				if (!isSorted(keys)) {
					System.out.println("Unsortierte Zahlenfolge nach Einfgen!");
					System.out.println(intArrayToString(keys));
					error = true;
				}
					
			}
			writeTree(tree);
		}
		if (!error){
			System.out.println("Zufall ok.");
		}
		System.out.println("++++++++++++++ Testlauf beendet ++++++++++++++");

		System.out.println("Gre des Baumes: " + tree.size());
		System.out.println("Ist der Baum leer?: " + tree.isEmpty());
		System.out.println("Hhe des Baumes: " + tree.height());
		System.out.println(
			"Ist Schlssel 1 im Baum?: " + tree.contains(1));
		try {
			System.out.print("Minimum im Baum: ");
			System.out.println(tree.minimum());
		} catch (EmptyTreeException e) {
			System.out.println(e.getMessage());
		}
		try {
			System.out.print("Maximum im Baum: ");
			System.out.println(tree.maximum());
		} catch (EmptyTreeException e) {
			System.out.println(e.getMessage());
		}
		System.out.println(
			"Zahlenfolge im Baum: "
				+ intArrayToString(tree.toIntArray()));
		System.out.println("XML-Ausgabe vom Baum: ");
		writeTree(tree);
		
	}
	
	
	/** Laufende Nummer fr die JPEG-Ausgabe */
	private static int fileCounter = 0;	
	
	/**
	 * Ausgabe des Baumes als XML.
	 * Es wird ein Output-Stream nach System.out
	 * sowie ein JPEG des Baumes erzeugt.
	 */
	public static void writeTree(MyTree tree) {
		Document document = tree.toXml();
		
		fileCounter ++;
		String filename = "output" + fileCounter;
		
		TransformerFactory f = TransformerFactory.newInstance();
		try {
			Transformer t = f.newTransformer();
			
			StreamResult rf = new StreamResult(filename + ".xml");
			StreamResult rc = new StreamResult(System.out);

			t.transform(new DOMSource(document),rf);
			t.transform(new DOMSource(document),rc);
			
			DrawTag dt = new DrawTag();
			dt.run(filename + ".xml",filename + ".jpg");

		} catch (Exception e) {
			e.printStackTrace();
		}
		System.out.println();
	}

	// Ausgabe eines integer-Arrays
	public static String intArrayToString(int[] array) {
		if (array.length == 0) {
			return "[]";
		}
		String s = "[" + array[0];
		for (int i = 1; i < array.length; ++i) {
			s = s + "," + array[i];
		}
		return s += "]";
	}

	// berprft, ob ein integer-Array streng aufsteigend sortiert
	// Fr so was wnscht man sich dann wieder haskell...
	public static boolean isSorted(int[] array) {
		for (int i = 1; i < array.length; ++i) {
			if (array[i] <= array[i - 1]) {
				return false;
			}
		}
		return true;
	}

	////////////////////// innere Klasse EmptyTreeException ////////////////////

	class EmptyTreeException extends Exception {
		public EmptyTreeException() {
			super();
		}
		public EmptyTreeException(String s) {
			super(s);
		}
	}
}
